/*
 Copyright (c) 2010 Myles Metzer

 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the "Software"), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
*/

#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/pgmspace.h>

#include "video_gen.h"
#include "spec/video_properties.h"
#include "spec/asm_macros.h"
#include "spec/hardware_setup.h"

//#define REMOVE6C
//#define REMOVE5C
//#define REMOVE4C
//#define REMOVE3C
#define byte uint8_t


unsigned char simbolLine;
int simbolInLine;
int addr;

const unsigned char *font;


TVout_vid display;
void (*render_line)();			//remove me
void (*line_handler)();			//remove me
void (*hbi_hook)() = &empty;
void (*vbi_hook)() = &empty;

// sound properties
volatile long remainingToneVsyncs;

void empty() {}

void render_setup(uint8_t *scrnptr, const unsigned char * f) {

	display.screen = scrnptr;
	display.hres = 16;//x;
	display.vres = 96;//y;
	display.frames = 0;
	//font = f + 32*8 +3;
  font = f+3;// + 32*8

		display.vscale_const = _PAL_LINE_DISPLAY/display.vres - 1;
	display.vscale = display.vscale_const;
	
	unsigned char rmethod = (_TIME_ACTIVE*_CYCLES_PER_US)/(display.hres*8);
	
	render_line = &render_lineAD;
	
	DDR_VID |= _BV(VID_PIN);
	DDR_SYNC |= _BV(SYNC_PIN);
	PORT_VID &= ~_BV(VID_PIN);
	PORT_SYNC |= _BV(SYNC_PIN);
	DDR_SND |= _BV(SND_PIN);	// for tone generation.
	
	// inverted fast pwm mode on timer 1
	TCCR1A = _BV(COM1A1) | _BV(COM1A0) | _BV(WGM11);
	TCCR1B = _BV(WGM13) | _BV(WGM12) | _BV(CS10);

		display.start_render = _PAL_LINE_MID - ((display.vres * (display.vscale_const+1))/2);
		display.output_delay = _PAL_CYCLES_OUTPUT_START;
		display.vsync_end = _PAL_LINE_STOP_VSYNC;
		display.lines_frame = _PAL_LINE_FRAME;
		ICR1 = _PAL_CYCLES_SCANLINE;
		OCR1A = _CYCLES_HORZ_SYNC;
	display.scanLine = display.lines_frame+1;
	line_handler = &vsync_line;
	TIMSK1 = _BV(TOIE1);
	sei();
}

// render a line
ISR(TIMER1_OVF_vect) {
	hbi_hook();
	line_handler();
}

void blank_line() {
		
	if ( display.scanLine == display.start_render) {
		display.vscale = display.vscale_const;
		line_handler = &active_line;
	}
	else if (display.scanLine == display.lines_frame) {
		line_handler = &vsync_line;
		vbi_hook();
	}
	
	display.scanLine++;
}

void active_line() {
	wait_until(display.output_delay);
	render_line();
	if (!display.vscale) {
		display.vscale = display.vscale_const;
	}
	else{
    simbolInLine = simbolInLine+1;
		if(simbolInLine==8){simbolInLine=0;simbolLine+=16;}
		//      
		addr=(int)font+simbolInLine;
		display.vscale--;
	}
		
	if ((display.scanLine + 1) == (int)(display.start_render + (display.vres*(display.vscale_const+1))))
		line_handler = &blank_line;
		
	display.scanLine++;
}

void vsync_line() {
	if (display.scanLine >= display.lines_frame) {
		OCR1A = _CYCLES_VIRT_SYNC;
		display.scanLine = 0;
		simbolLine = 0;
simbolInLine=0;
		display.frames++;

		if (remainingToneVsyncs != 0)
		{
			if (remainingToneVsyncs > 0)
			{
				remainingToneVsyncs--;
			}

		} else
		{
			TCCR2B = 0; //stop the tone
 			PORTB &= ~(_BV(SND_PIN));
		}

	}
	else if (display.scanLine == display.vsync_end) {
		OCR1A = _CYCLES_HORZ_SYNC;
		line_handler = &blank_line;
	}
	display.scanLine++;
}


static void inline wait_until(uint8_t time) {
	__asm__ __volatile__ (
			"subi	%[time], 10\n"
			"sub	%[time], %[tcnt1l]\n\t"
		"100:\n\t"
			"subi	%[time], 3\n\t"
			"brcc	100b\n\t"
			"subi	%[time], 0-3\n\t"
			"breq	101f\n\t"
			"dec	%[time]\n\t"
			"breq	102f\n\t"
			"rjmp	102f\n"
		"101:\n\t"
			"nop\n" 
		"102:\n"
		:
		: [time] "a" (time),
		[tcnt1l] "a" (TCNT1L)
	);
}


void render_lineAD(){
	__asm__ __volatile__ (
		"mov r30,r28\n\t"
		"mov r31,r29\n\t"
		"LD	r18,X+\n\t"
		"mov r19,r18\n\t"
    "lsl r18\n\t"//  3   
    "lsl r18\n\t"
    "lsl r18\n\t"
    "lsr r19\n\t"//  5   , ,       << 3
    "lsr r19\n\t"
    "lsr r19\n\t"
    "lsr r19\n\t"
    "lsr r19\n\t"
    "ADD r30,r18\n\t"
    "ADC r31,r19\n\t"
		//save PORTB
		"svprt	%[port]\n\t"
		"rjmp	enterAD\n\t"
	"loopAD:\n\t"
//    "mov r30,r28\n\t" //     ,  
//		"mov r31,r29\n\t"
//		"LD	r18,X+\n\t"
//		"mov r19,r18\n\t"
//    "lsl r18\n\t"//  3   
//    "lsl r18\n\t"
//    "lsl r18\n\t"
//    "lsr r19\n\t"//  5   , ,       << 3
//    "lsr r19\n\t"
//    "lsr r19\n\t"
//    "lsr r19\n\t"
//    "lsr r19\n\t"
//    "ADD r30,r18\n\t"
//    "ADC r31,r19\n\t"
		"bst	__tmp_reg__,0\n\t"			//8
		"o1bs	%[port]\n\t"
	"enterAD:\n\t"
//    "LD	__tmp_reg__,X+\n\t"
    "lpm\n\t"
		"bst	__tmp_reg__,7\n\t" //  T  7 
		"o1bs	%[port]\n\t"	//  r16  T
					//    r16
//		"delay2\n\t"						//2
    "mov r30,r28\n\t"
		"mov r31,r29\n\t"

		"bst	__tmp_reg__,6\n\t"
		"o1bs	%[port]\n\t"

//		"delay2\n\t"						//3
		"LD	r18,X+\n\t"
		"mov r19,r18\n\t"

		"bst	__tmp_reg__,5\n\t"
		"o1bs	%[port]\n\t"

//		"delay2\n\t"						//4
    "lsl r18\n\t"//  3   
    "lsl r18\n\t"

		"bst	__tmp_reg__,4\n\t"
		"o1bs	%[port]\n\t"

//		"delay2\n\t"						//5
    "lsl r18\n\t"
    "lsr r19\n\t"//  5   , ,       << 3

		"bst	__tmp_reg__,3\n\t"
		"o1bs	%[port]\n\t"

//		"delay2\n\t"						//6
    "lsr r19\n\t"
    "lsr r19\n\t"

		"bst	__tmp_reg__,2\n\t"
		"o1bs	%[port]\n\t"

//		"delay1\n\t"						//7
    "lsr r19\n\t"
    "lsr r19\n\t"
    "ADD r30,r18\n\t"
    "ADC r31,r19\n\t"

		"dec	%[hres]\n\t"
		"bst	__tmp_reg__,1\n\t"
		"o1bs	%[port]\n\t"
		"brne	loopAD\n\t"					//go too loopAD
		"delay1\n\t"
		"bst	__tmp_reg__,0\n\t"			//8
		"o1bs	%[port]\n"
		
		"svprt	%[port]\n\t"
		BST_HWS
		"o1bs	%[port]\n\t"
		:
		: [port] "i" (_SFR_IO_ADDR(PORT_VID)),
		"x" (display.screen+simbolLine),
		"y" (addr),
		[hres] "d" (display.hres)
		: "r16" // try to remove this clobber later...
	);
}

